@@ -33,6 +33,7 @@ Volume backups can be created, restored, deleted and listed.
 
 import contextlib
 import os
+import typing
 
 from castellan import key_manager
 from eventlet import tpool
@@ -139,6 +140,7 @@ class BackupManager(manager.SchedulerDependentManager):
         self.service = importutils.import_class(self.driver_name)
         self.message_api = message_api.API()
 
+    @typing.no_type_check
     def init_host(self, **kwargs):
         """Run initialization needed for a standalone service."""
         ctxt = context.get_admin_context()
@@ -211,7 +213,13 @@ class BackupManager(manager.SchedulerDependentManager):
                               "snapshots for backup %(bkup)s.",
                               {'bkup': backup['id']})
 
-    def _cleanup_one_volume(self, ctxt, volume):
+    def _cleanup_one_volume(self, ctxt, volume_id):
+        try:
+            volume = objects.Volume.get_by_id(ctxt, volume_id)
+        except exception.VolumeNotFound:
+            LOG.info('Volume %s does not exist anymore. Ignoring.', volume_id)
+            return
+
         if volume['status'] == 'backing-up':
             self._detach_all_attachments(ctxt, volume)
             LOG.info('Resetting volume %(vol_id)s to previous '
@@ -227,23 +235,38 @@ class BackupManager(manager.SchedulerDependentManager):
             self.db.volume_update(ctxt, volume['id'],
                                   {'status': 'error_restoring'})
 
+    def _cleanup_one_snapshot(self, ctxt, snapshot_id):
+        try:
+            snapshot = objects.Snapshot.get_by_id(ctxt, snapshot_id)
+        except exception.SnapshotNotFound:
+            LOG.info('Snapshot %s does not exist anymore. Ignoring.',
+                     snapshot_id)
+            return
+        if snapshot['status'] == 'backing-up':
+            LOG.info('Resetting snapshot %(snap_id)s to previous '
+                     'status %(status)s (was backing-up).',
+                     {'snap_id': snapshot['id'],
+                      'status': fields.SnapshotStatus.AVAILABLE})
+
+            snapshot.status = fields.SnapshotStatus.AVAILABLE
+            snapshot.save()
+
     def _cleanup_one_backup(self, ctxt, backup):
         if backup['status'] == fields.BackupStatus.CREATING:
             LOG.info('Resetting backup %s to error (was creating).',
                      backup['id'])
-
-            volume = objects.Volume.get_by_id(ctxt, backup.volume_id)
-            self._cleanup_one_volume(ctxt, volume)
-
+            self._cleanup_one_volume(ctxt, backup.volume_id)
+            if backup.snapshot_id:
+                self._cleanup_one_snapshot(ctxt, backup.snapshot_id)
             err = 'incomplete backup reset on manager restart'
             volume_utils.update_backup_error(backup, err)
         elif backup['status'] == fields.BackupStatus.RESTORING:
             LOG.info('Resetting backup %s to '
                      'available (was restoring).',
                      backup['id'])
-            volume = objects.Volume.get_by_id(ctxt, backup.restore_volume_id)
-            self._cleanup_one_volume(ctxt, volume)
-
+            self._cleanup_one_volume(ctxt, backup.restore_volume_id)
+            if backup.snapshot_id:
+                self._cleanup_one_snapshot(ctxt, backup.snapshot_id)
             backup.status = fields.BackupStatus.AVAILABLE
             backup.save()
         elif backup['status'] == fields.BackupStatus.DELETING:
@@ -405,6 +428,9 @@ class BackupManager(manager.SchedulerDependentManager):
                     detail=message_field.Detail.BACKUP_SERVICE_DOWN)
                 raise exception.InvalidBackup(reason=err)
 
+            if not backup.availability_zone:
+                backup.availability_zone = self.az
+
             backup.service = self.driver_name
             backup.save()
 
@@ -560,8 +586,8 @@ class BackupManager(manager.SchedulerDependentManager):
                                   {'status': previous_status,
                                    'previous_status': 'backing-up'})
 
-        # _run_backup method above updated the status for the backup, so it
-        # will reflect latest status, even if it is deleted
+        # continue_backup method above updated the status for the backup, so
+        # it will reflect latest status, even if it is deleted
         completion_msg = 'finished'
         if backup.status in (fields.BackupStatus.DELETING,
                              fields.BackupStatus.DELETED):
@@ -607,8 +633,15 @@ class BackupManager(manager.SchedulerDependentManager):
         return False
 
     @utils.limit_operations
-    def restore_backup(self, context, backup, volume_id):
-        """Restore volume backups from configured backup service."""
+    def restore_backup(self, context, backup, volume_id, volume_is_new):
+        """Restore volume backups from configured backup service.
+
+        :param context: RequestContext for the restore operation
+        :param backup: Backup that we're restoring
+        :param volume_id: The ID of the volume into which we're restoring
+        :param volume_is_new: The volume does not have stale data, so
+                              sparse backups can be restored as such.
+        """
         context.message_resource_id = backup.id
         context.message_resource_type = message_field.Resource.VOLUME_BACKUP
         context.message_action = message_field.Action.BACKUP_RESTORE
@@ -683,7 +716,7 @@ class BackupManager(manager.SchedulerDependentManager):
 
         canceled = False
         try:
-            self._run_restore(context, backup, volume)
+            self._run_restore(context, backup, volume, volume_is_new)
         except exception.BackupRestoreCancel:
             canceled = True
         except Exception:
@@ -725,7 +758,7 @@ class BackupManager(manager.SchedulerDependentManager):
                   'volume_id': volume_id})
         self._notify_about_backup_usage(context, backup, "restore.end")
 
-    def _run_restore(self, context, backup, volume):
+    def _run_restore(self, context, backup, volume, volume_is_new):
         message_created = False
         orig_key_id = volume.encryption_key_id
         backup_service = self.service(context)
@@ -754,16 +787,19 @@ class BackupManager(manager.SchedulerDependentManager):
                 if secure_enabled:
                     with open(device_path, open_mode) as device_file:
                         backup_service.restore(backup, volume.id,
-                                               tpool.Proxy(device_file))
+                                               tpool.Proxy(device_file),
+                                               volume_is_new)
                 else:
                     with utils.temporary_chown(device_path):
                         with open(device_path, open_mode) as device_file:
                             backup_service.restore(backup, volume.id,
-                                                   tpool.Proxy(device_file))
+                                                   tpool.Proxy(device_file),
+                                                   volume_is_new)
             # device_path is already file-like so no need to open it
             else:
                 backup_service.restore(backup, volume.id,
-                                       tpool.Proxy(device_path))
+                                       tpool.Proxy(device_path),
+                                       volume_is_new)
         except exception.BackupRestoreCancel:
             raise
         except Exception:
