@@ -28,7 +28,6 @@ import uuid
 import ddt
 from oslo_log import versionutils
 from oslo_utils import units
-import six
 
 from cinder import context
 from cinder import exception
@@ -136,22 +135,97 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
 
         self.library._create_lun.assert_called_once_with(
             fake.POOL_NAME, fake.LUN_NAME, volume_size_in_bytes,
-            fake.LUN_METADATA, fake.QOS_POLICY_GROUP_NAME, False)
+            fake.LUN_METADATA,
+            fake.QOS_POLICY_GROUP_NAME, False)
         self.library._get_volume_model_update.assert_called_once_with(
             fake.VOLUME)
         self.assertEqual(
             0, self.library. _mark_qos_policy_group_for_deletion.call_count)
         self.assertEqual(0, block_base.LOG.error.call_count)
 
+    def test_create_volume_space_allocation_extra_spec_false(self):
+        volume_size_in_bytes = int(fake.SIZE) * units.Gi
+        self.mock_object(na_utils, 'get_volume_extra_specs',
+                         return_value={
+                             'netapp:space_allocation': '<is> False'
+                         }
+                         )
+        self.mock_object(na_utils, 'log_extra_spec_warnings')
+        self.mock_object(block_base, 'LOG')
+        self.mock_object(volume_utils, 'extract_host',
+                         return_value=fake.POOL_NAME)
+        self.mock_object(self.library, '_setup_qos_for_volume',
+                         return_value=fake.QOS_POLICY_GROUP_INFO)
+        self.mock_object(self.library, '_create_lun')
+        self.mock_object(self.library, '_create_lun_handle')
+        self.mock_object(self.library, '_add_lun_to_table')
+        self.mock_object(self.library, '_mark_qos_policy_group_for_deletion')
+        self.mock_object(self.library, '_get_volume_model_update')
+
+        self.library.create_volume(fake.VOLUME)
+
+        self.library._create_lun.assert_called_once_with(
+            fake.POOL_NAME, fake.LUN_NAME, volume_size_in_bytes,
+            fake.LUN_METADATA,
+            fake.QOS_POLICY_GROUP_NAME, False)
+        self.library._get_volume_model_update.assert_called_once_with(
+            fake.VOLUME)
+        self.assertEqual(
+            0, self.library._mark_qos_policy_group_for_deletion.call_count)
+        self.assertEqual(0, block_base.LOG.error.call_count)
+
+    def test_create_volume_space_allocation_extra_spec_true(self):
+        volume_size_in_bytes = int(fake.SIZE) * units.Gi
+        self.mock_object(na_utils, 'get_volume_extra_specs',
+                         return_value={
+                             'netapp:space_allocation': '<is> True'
+                         }
+                         )
+        self.mock_object(na_utils, 'log_extra_spec_warnings')
+        self.mock_object(block_base, 'LOG')
+        self.mock_object(volume_utils, 'extract_host',
+                         return_value=fake.POOL_NAME)
+        self.mock_object(self.library, '_setup_qos_for_volume',
+                         return_value=fake.QOS_POLICY_GROUP_INFO)
+        self.mock_object(self.library, '_create_lun')
+        self.mock_object(self.library, '_create_lun_handle')
+        self.mock_object(self.library, '_add_lun_to_table')
+        self.mock_object(self.library, '_mark_qos_policy_group_for_deletion')
+        self.mock_object(self.library, '_get_volume_model_update')
+
+        self.library.create_volume(fake.VOLUME)
+
+        self.library._create_lun.assert_called_once_with(
+            fake.POOL_NAME, fake.LUN_NAME, volume_size_in_bytes,
+            fake.LUN_METADATA_WITH_SPACE_ALLOCATION,
+            fake.QOS_POLICY_GROUP_NAME, False)
+        self.library._get_volume_model_update.assert_called_once_with(
+            fake.VOLUME)
+        self.assertEqual(
+            0, self.library._mark_qos_policy_group_for_deletion.call_count)
+        self.assertEqual(0, block_base.LOG.error.call_count)
+
     def test_create_volume_no_pool(self):
         self.mock_object(volume_utils, 'extract_host', return_value=None)
 
         self.assertRaises(exception.InvalidHost, self.library.create_volume,
                           fake.VOLUME)
 
+    def test_space_allocation_exception_path(self):
+        self.mock_object(block_base, 'LOG')
+        self.mock_object(na_utils, 'get_volume_extra_specs',
+                         return_value={'netapp:space_allocation': 'xyz'})
+        self.mock_object(self.library, '_setup_qos_for_volume',
+                         return_value=fake.QOS_POLICY_GROUP_INFO)
+        self.mock_object(self.library, '_create_lun', side_effect=Exception)
+        self.mock_object(self.library, '_mark_qos_policy_group_for_deletion')
+        self.assertRaises(exception.VolumeBackendAPIException,
+                          self.library.create_volume, fake.VOLUME)
+
     def test_create_volume_exception_path(self):
         self.mock_object(block_base, 'LOG')
-        self.mock_object(na_utils, 'get_volume_extra_specs')
+        self.mock_object(na_utils, 'get_volume_extra_specs',
+                         return_value={})
         self.mock_object(self.library, '_setup_qos_for_volume',
                          return_value=fake.QOS_POLICY_GROUP_INFO)
         self.mock_object(self.library, '_create_lun', side_effect=Exception)
@@ -767,6 +841,9 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
         self.mock_object(block_base.NetAppBlockStorageLibrary,
                          '_get_targets_from_list',
                          return_value=target_details_list)
+        self.mock_object(block_base.NetAppBlockStorageLibrary,
+                         '_is_space_alloc_enabled',
+                         return_value=True)
         self.zapi_client.get_iscsi_service_details.return_value = (
             fake.ISCSI_SERVICE_IQN)
         self.mock_object(na_utils,
@@ -796,6 +873,10 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
             fake.ISCSI_CONNECTION_PROPERTIES['data']
             ['discovery_auth_username'],
             target_info['data']['discovery_auth_username'])
+        self.assertEqual(
+            fake.ISCSI_CONNECTION_PROPERTIES['data']
+            ['discard'],
+            target_info['data']['discard'])
 
         self.assertEqual(fake.ISCSI_CONNECTION_PROPERTIES, target_info)
         block_base.NetAppBlockStorageLibrary._map_lun.assert_called_once_with(
@@ -815,6 +896,9 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
         self.zapi_client.get_iscsi_target_details.return_value = None
         self.mock_object(block_base.NetAppBlockStorageLibrary,
                          '_get_targets_from_list')
+        self.mock_object(block_base.NetAppBlockStorageLibrary,
+                         '_is_space_alloc_enabled',
+                         return_value=True)
         self.mock_object(na_utils,
                          'get_iscsi_connection_properties',
                          return_value=fake.ISCSI_CONNECTION_PROPERTIES)
@@ -826,6 +910,9 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
         self.assertEqual(
             0, block_base.NetAppBlockStorageLibrary
                          ._get_targets_from_list.call_count)
+        self.assertEqual(
+            0, block_base.NetAppBlockStorageLibrary
+            ._is_space_alloc_enabled.call_count)
         self.assertEqual(
             0, self.zapi_client.get_iscsi_service_details.call_count)
         self.assertEqual(
@@ -840,6 +927,9 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
         self.mock_object(block_base.NetAppBlockStorageLibrary,
                          '_get_targets_from_list',
                          return_value=None)
+        self.mock_object(block_base.NetAppBlockStorageLibrary,
+                         '_is_space_alloc_enabled',
+                         return_value=True)
         self.mock_object(na_utils, 'get_iscsi_connection_properties')
 
         self.assertRaises(exception.VolumeBackendAPIException,
@@ -848,6 +938,9 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
 
         self.assertEqual(0, self.zapi_client
                                 .get_iscsi_service_details.call_count)
+        self.assertEqual(0,
+                         block_base.NetAppBlockStorageLibrary
+                         ._is_space_alloc_enabled.call_count)
         self.assertEqual(0, na_utils.get_iscsi_connection_properties
                                     .call_count)
 
@@ -1235,7 +1328,7 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
                                         fake.LUN_METADATA)
         mock_get_lun_from_table = self.mock_object(
             self.library, '_get_lun_from_table', return_value=fake_lun)
-        fake_lun_geometry = {'max_resize': six.text_type(max_size_bytes)}
+        fake_lun_geometry = {'max_resize': str(max_size_bytes)}
         mock_get_lun_geometry = self.mock_object(
             self.library.zapi_client, 'get_lun_geometry',
             return_value=fake_lun_geometry)
@@ -1257,9 +1350,9 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
             mock_get_lun_geometry.assert_not_called()
 
         mock_do_direct_resize.assert_called_once_with(
-            fake.LUN_METADATA['Path'], six.text_type(new_size_bytes))
+            fake.LUN_METADATA['Path'], str(new_size_bytes))
         self.assertFalse(mock_do_sub_clone_resize.called)
-        self.assertEqual(six.text_type(new_size_bytes),
+        self.assertEqual(str(new_size_bytes),
                          self.library.lun_table[fake.VOLUME['name']].size)
 
     @ddt.data((9, 4, 0), (9, 6, 0))
@@ -1284,7 +1377,7 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
                                         fake.LUN_METADATA)
         mock_get_lun_from_table = self.mock_object(
             self.library, '_get_lun_from_table', return_value=fake_lun)
-        fake_lun_geometry = {'max_resize': six.text_type(max_size_bytes)}
+        fake_lun_geometry = {'max_resize': str(max_size_bytes)}
         mock_get_lun_geometry = self.mock_object(
             self.library.zapi_client, 'get_lun_geometry',
             return_value=fake_lun_geometry)
@@ -1306,9 +1399,9 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
             mock_get_lun_geometry.assert_not_called()
 
         mock_do_direct_resize.assert_called_once_with(
-            fake.LUN_METADATA['Path'], six.text_type(new_size_bytes))
+            fake.LUN_METADATA['Path'], str(new_size_bytes))
         self.assertFalse(mock_do_sub_clone_resize.called)
-        self.assertEqual(six.text_type(new_size_bytes),
+        self.assertEqual(str(new_size_bytes),
                          self.library.lun_table[volume_copy['name']].size)
 
     @ddt.data((9, 4, 0), (9, 6, 0))
@@ -1330,7 +1423,7 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
                                         fake.LUN_METADATA)
         mock_get_lun_from_table = self.mock_object(
             self.library, '_get_lun_from_table', return_value=fake_lun)
-        fake_lun_geometry = {'max_resize': six.text_type(max_size_bytes)}
+        fake_lun_geometry = {'max_resize': str(max_size_bytes)}
         mock_get_lun_geometry = self.mock_object(
             self.library.zapi_client, 'get_lun_geometry',
             return_value=fake_lun_geometry)
@@ -1350,15 +1443,15 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
             mock_get_lun_geometry.assert_called_once_with(
                 fake.LUN_METADATA['Path'])
             mock_do_sub_clone_resize.assert_called_once_with(
-                fake.LUN_METADATA['Path'], six.text_type(new_size_bytes),
+                fake.LUN_METADATA['Path'], str(new_size_bytes),
                 qos_policy_group_name='fake_qos_policy')
         else:
             mock_get_lun_geometry.assert_not_called()
             mock_do_sub_clone_resize.assert_not_called()
             mock_do_direct_resize.assert_called_once_with(
-                fake.LUN_METADATA['Path'], six.text_type(new_size_bytes))
+                fake.LUN_METADATA['Path'], str(new_size_bytes))
 
-        self.assertEqual(six.text_type(new_size_bytes),
+        self.assertEqual(str(new_size_bytes),
                          self.library.lun_table[fake.VOLUME['name']].size)
 
     @ddt.data((9, 4, 0), (9, 6, 0))
@@ -1378,11 +1471,11 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
             return_value=ontap_version)
         fake_lun = block_base.NetAppLun(fake.LUN_HANDLE,
                                         fake.LUN_ID,
-                                        six.text_type(current_size_bytes),
+                                        str(current_size_bytes),
                                         fake.LUN_METADATA)
         mock_get_lun_from_table = self.mock_object(
             self.library, '_get_lun_from_table', return_value=fake_lun)
-        fake_lun_geometry = {'max_resize': six.text_type(max_size_bytes)}
+        fake_lun_geometry = {'max_resize': str(max_size_bytes)}
         mock_get_lun_geometry = self.mock_object(
             self.library.zapi_client, 'get_lun_geometry',
             return_value=fake_lun_geometry)
@@ -1403,16 +1496,16 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
             self.assertFalse(mock_do_sub_clone_resize.called)
             mock_get_lun_geometry.assert_called_once_with(
                 fake.LUN_METADATA['Path'])
-            self.assertEqual(six.text_type(current_size_bytes),
+            self.assertEqual(str(current_size_bytes),
                              self.library.lun_table[volume_copy['name']].size)
         else:
             self.library._extend_volume(volume_copy,
                                         new_size, fake.QOS_POLICY_GROUP_NAME)
             mock_do_direct_resize.assert_called_once_with(
-                fake.LUN_METADATA['Path'], six.text_type(new_size_bytes))
+                fake.LUN_METADATA['Path'], str(new_size_bytes))
             mock_do_sub_clone_resize.assert_not_called()
             mock_get_lun_geometry.assert_not_called()
-            self.assertEqual(six.text_type(new_size_bytes),
+            self.assertEqual(str(new_size_bytes),
                              self.library.lun_table[volume_copy['name']].size)
 
         mock_get_ontap_version.assert_called_once_with(cached=True)
@@ -1438,7 +1531,7 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
                                         fake.LUN_METADATA)
         mock_get_lun_from_table = self.mock_object(
             self.library, '_get_lun_from_table', return_value=fake_lun)
-        fake_lun_geometry = {'max_resize': six.text_type(max_size_bytes)}
+        fake_lun_geometry = {'max_resize': str(max_size_bytes)}
         mock_get_lun_geometry = self.mock_object(
             self.library.zapi_client, 'get_lun_geometry',
             return_value=fake_lun_geometry)
