@@ -15,17 +15,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import sys
-
 from oslo_log import log as logging
 from oslo_utils import excutils
-import six
 
 from cinder import exception
 from cinder.i18n import _
 from cinder import utils
 from cinder.volume.drivers.netapp.dataontap.client import api as netapp_api
 from cinder.volume.drivers.netapp import utils as na_utils
+from cinder.volume import volume_utils
 
 LOG = logging.getLogger(__name__)
 
@@ -33,8 +31,7 @@ DELETED_PREFIX = 'deleted_cinder_'
 MAX_SIZE_FOR_A_LUN = '17555678822400'
 
 
-@six.add_metaclass(utils.TraceWrapperMetaclass)
-class Client(object):
+class Client(object, metaclass=volume_utils.TraceWrapperMetaclass):
 
     def __init__(self, **kwargs):
         host = kwargs['hostname']
@@ -62,25 +59,26 @@ class Client(object):
         self.features = na_utils.Features()
 
     def get_ontap_version(self, cached=True):
-        """Gets the ONTAP version."""
+        """Gets the ONTAP version-string and version-tuple"""
 
         if cached:
             return self.connection.get_ontap_version()
 
         ontap_version = netapp_api.NaElement("system-get-version")
-        result = self.connection.invoke_successfully(ontap_version, True)
+        result = self.connection.invoke_successfully(
+            ontap_version, enable_tunneling=True)
 
         version_tuple = result.get_child_by_name(
             'version-tuple') or netapp_api.NaElement('none')
-        system_version_tuple = version_tuple.get_child_by_name(
+        ontap_version_tuple = version_tuple.get_child_by_name(
             'system-version-tuple') or netapp_api.NaElement('none')
 
-        generation = system_version_tuple.get_child_content("generation")
-        major = system_version_tuple.get_child_content("major")
+        version = (
+            int(ontap_version_tuple.get_child_content('generation')),
+            int(ontap_version_tuple.get_child_content('major')),
+            int(ontap_version_tuple.get_child_content('minor')))
 
-        return '%(generation)s.%(major)s' % {
-            'generation': generation,
-            'major': major}
+        return version
 
     def get_ontapi_version(self, cached=True):
         """Gets the supported ontapi version."""
@@ -108,6 +106,7 @@ class Client(object):
                    qos_policy_group_name=None,
                    qos_policy_group_is_adaptive=False):
         """Issues API request for creating LUN on volume."""
+        self._validate_qos_policy_group(qos_policy_group_is_adaptive)
 
         path = '/vol/%s/%s' % (volume_name, lun_name)
         space_reservation = metadata['SpaceReserved']
@@ -118,7 +117,7 @@ class Client(object):
         # geometry on max_resize_size. In order to remove this
         # limitation we create the LUN with its maximum possible size
         # and then shrink to the requested size.
-        if ontap_version < '9.5':
+        if ontap_version < (9, 5, 0):
             initial_size = MAX_SIZE_FOR_A_LUN
             # In order to create a LUN with its maximum size (16TB),
             # the space_reservation needs to be disabled
@@ -127,6 +126,10 @@ class Client(object):
         params = {'path': path, 'size': str(initial_size),
                   'ostype': metadata['OsType'],
                   'space-reservation-enabled': space_reservation}
+
+        if "SpaceAllocated" in metadata:
+            params['space-allocation-enabled'] = metadata['SpaceAllocated']
+
         version = self.get_ontapi_version()
         if version >= (1, 110):
             params['use-exact-size'] = 'true'
@@ -151,8 +154,8 @@ class Client(object):
                            'volume_name': volume_name,
                            'ex': ex})
 
-        if ontap_version < '9.5':
-            self.do_direct_resize(path, six.text_type(size))
+        if ontap_version < (9, 5, 0):
+            self.do_direct_resize(path, str(size))
             if metadata['SpaceReserved'] == 'true':
                 self.set_lun_space_reservation(path, True)
 
@@ -202,7 +205,6 @@ class Client(object):
         try:
             self.connection.invoke_successfully(lun_unmap, True)
         except netapp_api.NaApiError as e:
-            exc_info = sys.exc_info()
             LOG.warning("Error unmapping LUN. Code :%(code)s, Message: "
                         "%(message)s", {'code': e.code,
                                         'message': e.message})
@@ -210,7 +212,7 @@ class Client(object):
             if e.code == '13115' or e.code == '9016':
                 pass
             else:
-                six.reraise(*exc_info)
+                raise e
 
     def create_igroup(self, igroup, igroup_type='iscsi', os_type='default'):
         """Creates igroup with specified args."""
@@ -260,7 +262,7 @@ class Client(object):
                 "max-resize-size")
         except Exception as e:
             LOG.error("LUN %(path)s geometry failed. Message - %(msg)s",
-                      {'path': path, 'msg': six.text_type(e)})
+                      {'path': path, 'msg': str(e)})
         return geometry
 
     def get_volume_options(self, volume_name):
@@ -313,6 +315,10 @@ class Client(object):
         """Get igroups exactly matching a set of initiators."""
         raise NotImplementedError()
 
+    def _validate_qos_policy_group(self, is_adaptive, spec=None, is_nfs=False):
+        """Raises an exception if the backend doesn't support the QoS spec."""
+        raise NotImplementedError()
+
     def _has_luns_mapped_to_initiator(self, initiator):
         """Checks whether any LUNs are mapped to the given initiator."""
         lun_list_api = netapp_api.NaElement('lun-initiator-list-map-info')
